@@ -13,7 +13,7 @@
 ##' @note The default ugarchspec.list fits an ARMA(1,1)-GARCH(1,1) with N(0,1)
 ##'       standardized residuals
 fit_ARMA_GARCH <- function(x, ugarchspec.list = ugarchspec(), solver = "hybrid",
-                           verbose = TRUE, ...)
+                           verbose = FALSE, ...)
 {
     ## Checking and expanding ugarchspec.list to a list of length d
     if(!is.matrix(x)) x <- cbind(x) # is.matrix() is also true for 'xts' objects
@@ -28,17 +28,23 @@ fit_ARMA_GARCH <- function(x, ugarchspec.list = ugarchspec(), solver = "hybrid",
     fit  <- vector("list", length = d)
     warn <- vector("list", length = d)
     err  <- vector("list", length = d)
-    if(verbose) {
-        pb <- txtProgressBar(max = d, style = if(isatty(stdout())) 3 else 1)
-        on.exit(close(pb)) # on exit, close progress bar
-    }
+    ## if(verbose) {
+    ##     pb <- txtProgressBar(max = d, style = if(isatty(stdout())) 3 else 1)
+    ##     on.exit(close(pb)) # on exit, close progress bar
+    ## }
     for(j in seq_len(d)) {
         res <- catch(ugarchfit(ugarchspec.list[[j]], data = x[,j], solver = solver,
                                ...)) # fitting
         if(!is.null(res$value)) fit[[j]] <- res$value
         if(!is.null(res$warning)) warn[[j]] <- res$warning
         if(!is.null(res$error)) err[[j]]  <- res$error
-        if(verbose) setTxtProgressBar(pb, j) # update progress bar
+        ## Progress
+        if(verbose) {
+            ## setTxtProgressBar(pb, j) # update progress bar
+            if(j %% ceiling(d/20) == 0)
+                cat(sprintf("%2d%% ", ceiling(j/d * 100)))
+            if(j == d) cat("\n")
+        }
     }
 
     ## Return
@@ -193,3 +199,31 @@ fit_GARCH_11 <- function(x, init = NULL, # z_{cor}, z_{ema}, (d.o.f. nu)
          sig.t = sqrt(sig2.t), # conditional volatility \sigma_t
          Z.t = Z.t) # standardized residuals Z_t
 }
+
+##' @title GARCH(1,1) Tail Index
+##' @param innovations realizations of the innovations Z to estimate the mean
+##'        (obtained, e.g., via rnorm(n) or rt(n, df = nu) * sqrt((nu-2)/nu))
+##' @param alpha1 GARCH(1,1) coefficient, >= 0 with alpha1 + beta1 < 1
+##' @param beta1 GARCH(1,1) coefficient, >= 0 with alpha1 + beta1 < 1
+##' @param interval initial interval for root finding
+##' @param ... additional arguments passed to the underlying uniroot()
+##' @return approximate tail index alpha (or NA if not found)
+##' @author Marius Hofert
+##' @note - E((alpha_1 * Z^2 + beta_1)^(alpha/2)) = 1 according to
+##'         McNeil et al. (2015, p. 576); see also p. 118, Definition 4.20,
+##'         p. 119, Proposition 4.21
+##'       - For checking:
+##'         a <- seq(0, 10, length.out = 101)
+##'         y <- sapply(a, function(x) f(x))
+##'         plot(a, y, type = "l")
+##'       - Careful: The tail index formula implemented assumes include.mean = FALSE
+##'         (so the GARCH(1,1) has no additional constant mean).
+tail_index_GARCH_11 <- function(innovations, alpha1, beta1,
+                                interval = c(1e-6, 1e2), ...)
+{
+    if(!(alpha1 >= 0 && beta1 >= 0 && alpha1 + beta1 < 1)) NA
+    f <- function(a) mean((alpha1 * innovations^2 + beta1)^(a/2)) - 1
+    f. <- f(interval[1])
+    f.. <- f(interval[2])
+    if(f. * f.. >= 0) NA else uniroot(f, interval = interval, ...)$root
+}
